/***********************************************************************
               F.W.BELL SERIES-9900 / MODEL-9500 GAUSSMETER
                      IEEE-488 DEMONSTATION PROGRAM
************************************************************************/
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <bios.h>
#include <string.h>
#pragma inline

/* Gaussmeter command strings */

char MEASURE1[12]    = {"ME100000000"};         /* measure */
char MODE1[5]        = {"MO12"};                /* mode select (Gauss DC) */
char RANGE1[5]       = {"RA12"};                /* range select (30 G) */
char PEAK1_OFF[5]    = {"PE11"};                /* peak hold off */
char DISPLAY1[8]     = {"DI1222"};              /* display/bargraph/lite on */

/* Metrabyte command strings */

char    *command_string[18] = {"ABORT","CLEAR ","CONFIG ","ENTER ","EOI ",
                               "LOCAL ","LOCKOUT ","OUTPUT ","PARPOL",
                               "PASCTL ","PPCONF ","PPUNCF ","REMOTE ",
                               "REQUEST ","RXCTL","STATUS ","SYSCON ",
                               "TRIGGER "};

/* Metrabyte hardware setup */

char    *syscon_string[4] = {"[$%,0,",                   /* image specifier */
                            "MAD=00,CIC=1,NOB=1,BA0=&H300",
                            "TALK=",
                            "LISTEN="};

unsigned long IEEE_rom_seg = 0xC0000000;
unsigned int  IEEE_board_number = 0;

unsigned char IEEE_cmd_buf[50],      *IEEE_cmd_buf_pointer;
unsigned char GM_IO_buf[256],        *GM_IO_buf_pointer;

unsigned char IEEE_cmd_descr[3];
unsigned char GM_IO_descr[3];
unsigned int  IEEE_error = 0;

#define ABORT             0
#define CLEAR             1
#define CONFIG            2
#define ENTER             3
#define EOI               4
#define LOCAL             5
#define LOCKOUT           6
#define OUTPUT            7
#define PARPOL            8
#define PASCTL            9
#define PPCONF           10
#define PPUNCF           11
#define REMOTE           12
#define REQUEST          13
#define RXCTL            14
#define STATUS           15
#define SYSCON           16
#define TRIGGER          17

/* miscellaneous definitions */

#define  ESC             27           /* <ESC> */
#define  CR              13           /* <CR>  */
#define  YES              1
#define  NO               0
#define  GM_ADDR         15
#define  GM_SRQ        0x40
#define  GM_RDY        0x00
#define  GM_BUSY       0x01

int     outlen, inlen;

char    ascii_buffer[3];

/* prototype functions */

void GM_setup();
void report_error();
void IEEE_488();
void unsigned_byte_to_ASCII();

int main ()
{
    clrscr();
    printf("************************************************************************\n");
    printf("               F.W.BELL SERIES-9900 / MODEL-9500 GAUSSMETER\n");
    printf("                     IEEE-488 DEMONSTATION PROGRAM\n");
    printf("************************************************************************\n");
    printf("This program was written to assist users of a SERIES-9900 or MODEL-9500\n");
    printf(" Gaussmeter who wish to control the instrument remotely via the \n");
    printf(" IEEE-488 instrumentation bus.  This program was compiled and tested on \n");
    printf(" an IBM PC compatible using the Borland TURBO-C++ compiler.  The intent\n");
    printf(" of this program is to provide the systems programmer with a template to \n");
    printf(" use during the development of his or her own code.  This program requires\n");
    printf(" that the Gaussmeter contain V1.7A or higher software.  The software \n");
    printf(" version number is displayed immediately after power-up.\n\n");

    printf("Press any key to continue.....");

    while (kbhit() != 0)  {getch();}
    while (kbhit() == 0);  getch(); clrscr();

    printf("The program is designed to operate a METRABYTE IE-488 controller card\n");
    printf(" configured for a memory address of C0000 hex, and I/O base address\n");
    printf(" of 300 hex, DMA channel-3 and INTERRUPT-5.  The GAUSSMETER's device\n");
    printf(" address should be set to 15 decimal using the COMMUNICATIONS FORMAT\n");
    printf(" menu.  To change these parameters see the METRABYTE HARDWARE SETUP\n");
    printf(" definitions in the program and the METRABYTE user's manual.\n\n");

    printf("The program will begin by programming the Gaussmeter's MODE for GAUSS-DC,\n");
    printf(" the RANGE for 30 G, PEAK HOLD off and DISPLAY DIGITS, BARGRAPH and BACK-\n");
    printf(" on.  After this field measurements will be aquired and displayed until the\n");
    printf(" user presses any key.  The program will work with a MODEL-9500 or with \n");
    printf(" CHANNEL-1 on a SERIES-9900\n\n");

    printf("Press any key to start.....");

    while (kbhit() == 0);  getch(); clrscr();

    /* Initialize serial comm Hardware */

    printf("Initializing IEEE-488 bus.\n");
    IEEE_488(SYSCON,3,1);
    printf("Locking out front panel on Gaussmeter.\n");
    IEEE_488(REMOTE,GM_ADDR,1);

    /* Issue setup commands to Gaussmeter */

    printf("Sending MODE command to Gaussmeter...\n");
    GM_setup(&MODE1[0], YES);
    printf("Sending RANGE command to Gaussmeter...\n");
    GM_setup(&RANGE1[0], YES);
    printf("Sending PEAK HOLD command to Gaussmeter...\n");
    GM_setup(&PEAK1_OFF[0], YES);
    printf("Sending DISPLAY FORMAT command to Gaussmeter...\n");
    GM_setup(&DISPLAY1[0], YES);
    delay(2000);

    /* Aquire field measurements */

    clrscr();
    printf("Sending MEASURE commands to Gaussmeter...\n");
    printf("     Press any key to terminate.\n");
    while (kbhit() == 0) {GM_setup(&MEASURE1[0], NO);}

    /* Terminate session */ 

    printf("Unlocking front panel on Gaussmeter.\n");
    IEEE_488(LOCAL,GM_ADDR,1);
    delay(2000);
    return(0);
}
/***********************************************************************/
void GM_setup(command_string,compare)
/***********************************************************************
This function prepares for the transmission of a command string to the
Gaussmeter and processes the response sent back.

Upon entry <command_string> should point to the ASCII command string to 
be transmitted.  Also, <compare> should be set to "1" if the caller 
wishes to have the entire command string compared to the response or 
"0" to compare just the first three characters of the response.  The 
latter should be used when issuing the MEASURE, RELATIVE-ON or ZERO 
command strings because the remaining characters in the response vary.

If the number of characters in the response is incorrect, or the 
contents of the response is incorrect, the command will be reissued
until all is well.

Upon exit the "GM_IO_buf" buffer will contain the ASCII response sent
back by the Gaussmeter (not including the <CR>) terminated with a <NULL> 
character. 

Also upon exit the global variable "outlen" will indicate the total 
number of characters transmitted including the <ESC> and <CR> characters.
************************************************************************/
unsigned char *command_string, compare;
{
	int match,strcmp(),strncmp();

do  {
    /* Wait for Gaussmeter to be idle */
      
        do {IEEE_488(STATUS,GM_ADDR,1);}  while (GM_IO_descr[0] != GM_RDY);
      
    /* Build Gaussmeter command string */
    
        GM_IO_buf_pointer = GM_IO_buf;
        *GM_IO_buf_pointer++ = ESC;
        GM_IO_buf_pointer = 
              stpcpy(GM_IO_buf_pointer,command_string);
        *GM_IO_buf_pointer++ = CR;
        outlen = GM_IO_buf_pointer - GM_IO_buf;
    
    /* Output command string and trigger Gaussmeter */ 
    
        IEEE_488(OUTPUT,GM_ADDR,outlen);
        printf("COMMAND SENT WAS [%s]\t\t",command_string);
    
        IEEE_488(TRIGGER,GM_ADDR,1);
    
    /* Wait for Gaussmeter to indicate that response in ready */
      
        do {IEEE_488(STATUS,GM_ADDR,1);}  while (GM_IO_descr[0] != GM_SRQ);
      
    /* Get Gaussmeter response */
    
        IEEE_488(ENTER,GM_ADDR,50);
        GM_IO_buf_pointer = GM_IO_buf;
        while (*GM_IO_buf_pointer++ != CR);
        inlen = (GM_IO_buf_pointer - GM_IO_buf) + 1;
        *--GM_IO_buf_pointer = 0;
        printf("GAUSSMETER RESPONSE WAS [%s]\n",GM_IO_buf);

        if(compare == YES) {match = strcmp(command_string,GM_IO_buf);}
        else               {match = strncmp(command_string,GM_IO_buf,3);}
    }
    while ((inlen != outlen) || (match != 0));
}
/***********************************************************************/
void IEEE_488(command,device,count)
/************************************************************************
This routine performs all communications with the IEEE-488 bus through
the METRABYTE MODEL IE-488 GPIB controller card.    The format of the 
call to this routine is:

                    IEEE_488(command,device,count),   where:

            command = an 8-bit number indicating the command, as follows:
                           (see the METRABYTE manual for details)

                        00 - ABORT    01 - CLEAR    02 - CONFIG
                        03 - ENTER    04 - EOI      05 - LOCAL
                        06 - LOCKOUT  07 - OUTPUT   08 - PARPOL
                        09 - PASCTL   10 - PPCONF   11 - PPUNCF
                        12 - REMOTE   13 - REQUEST  14 - RXCTL
                        15 - STATUS   16 - SYSCON   17 - TRIGGER

            device = an 8-bit number corresponding to the device's address
                           on the IEEE-488 bus.  NOTE: The METRABYTE only 
                           supports addresses 0 - 30 (decimal).

                        NOTE: For the SYSCON command, "device" will be the
                              device address programmed for the IBM.

            count = an 8-bit number specifying the number of bytes to
                        input/output to/from the "GM_IO_buf" data
                        buffer.

                        NOTE: For the CONFIG command,"count" serves as a
                                flag to indicate whether the device is 
                                programmed as a TALKER (count = 0) or
                                LISTENER (count > 0).

NOTE: For the OUTPUT command the system data buffer "GM_IO_buf[]"
      must be initialized with output data BEFORE calling this routine.

      For the ENTER command, "GM_IO_buf[] will be used to hold data
      returned from the device.
      
      For the STATUS command the status byte is returned in the 
      "GM_IO_descr[0]" variable.

If any errors occur the METRABYTE ERROR CODE will be reported (except for
the timeout error, 0x0200) and the command will be reissued until all
is well.
*************************************************************************/
    char    command, device, count;
{
    char    *command_string_pointer,*stpcpy();

    /* initialize command buffer and pointers */

    do{
        strset(IEEE_cmd_buf,0);
        unsigned_byte_to_ASCII(device);
        command_string_pointer = command_string[command];
        IEEE_cmd_buf_pointer = IEEE_cmd_buf;
        
        /* load command string into command buffer */
    
        IEEE_cmd_buf_pointer = 
              stpcpy(IEEE_cmd_buf_pointer,command_string_pointer);
    
        /* finish command buffer */
    
        switch ( command )
        {
            case CLEAR:  case LOCAL:  case LOCKOUT:
            case PASCTL: case PPCONF: case PPUNCF:
            case REMOTE: case STATUS: case TRIGGER:
    
                    /* load in device address */
    
                    *IEEE_cmd_buf_pointer++ = ascii_buffer[1];
                    *IEEE_cmd_buf_pointer++ = ascii_buffer[2];
                    break;
    
            case ENTER:  case EOI:  case OUTPUT:
    
                    /* load in device address */
    
                    *IEEE_cmd_buf_pointer++ = ascii_buffer[1];
                    *IEEE_cmd_buf_pointer++ = ascii_buffer[2];
    
                    /* load in first part of image specifier */

                    IEEE_cmd_buf_pointer = 
                        stpcpy(IEEE_cmd_buf_pointer,syscon_string[0]);
    
                    /* convert count to ascii and load it */
    
                    unsigned_byte_to_ASCII(count-1);
                    *IEEE_cmd_buf_pointer++ = ascii_buffer[0];    
                    *IEEE_cmd_buf_pointer++ = ascii_buffer[1];    
                    *IEEE_cmd_buf_pointer++ = ascii_buffer[2];
                    
                    /* finish command string */
    
                    *IEEE_cmd_buf_pointer++ = ']';
                    break;
    
            case SYSCON:
    
                    /* load command */
                    IEEE_cmd_buf_pointer =
                        stpcpy(IEEE_cmd_buf_pointer,syscon_string[1]);

                    /* fill in board number */
    
                    IEEE_cmd_buf[11] = ascii_buffer[1];                
                    IEEE_cmd_buf[12] = ascii_buffer[2];
    
                    break;
    
            case CONFIG:
    
                    /* load first part of command */
                    if(count == 0)
                    {
                        IEEE_cmd_buf_pointer = 
                            stpcpy(IEEE_cmd_buf_pointer,syscon_string[2]);
                    }
                    else
                    {
                        IEEE_cmd_buf_pointer =
                            stpcpy(IEEE_cmd_buf_pointer,syscon_string[3]);
                    }
                    *IEEE_cmd_buf_pointer++ = ascii_buffer[1];                
                    *IEEE_cmd_buf_pointer++ = ascii_buffer[2];
                    break;
        }

        /* load descriptor buffer for the call */

        IEEE_cmd_descr[0] = (unsigned char)(IEEE_cmd_buf_pointer - IEEE_cmd_buf);
        GM_IO_descr[0] = count;
 
asm{
    PUSH    AX
    PUSH    BX
    PUSH    CX
    PUSH    DX
    PUSH    BP
    PUSH    SI
    PUSH    DI

    MOV     SI,OFFSET BYTE PTR IEEE_cmd_descr[1]
    MOV     AX,OFFSET BYTE PTR IEEE_cmd_buf[0]
    MOV     WORD PTR [SI],AX
    MOV     SI,OFFSET BYTE PTR GM_IO_descr[1]
    MOV     AX,OFFSET BYTE PTR GM_IO_buf[0]
    MOV     WORD PTR [SI],AX
    MOV     AX,OFFSET BYTE PTR IEEE_cmd_descr[0]
    PUSH    AX
    MOV     AX,OFFSET BYTE PTR GM_IO_descr[0]
    PUSH    AX
    MOV     AX,OFFSET WORD PTR IEEE_error
    PUSH    AX
    MOV     AX,OFFSET WORD PTR IEEE_board_number
    PUSH    AX

    CALL    DWORD PTR IEEE_rom_seg

    POP     DI
    POP     SI
    POP     BP
    POP     DX
    POP     CX
    POP     BX
    POP     AX
}
        if(IEEE_error != 0 && IEEE_error != 0x0200) {report_error();}
    }
    while(IEEE_error != 0);
}
/***********************************************************************/
void unsigned_byte_to_ASCII( number )
/************************************************************************
This routine converts an unsigned 8-bit <number> to a 3-digit ASCII 
string.  The three digits are stored in the three bytes of the system 
buffer "ascii_buffer[]". 
************************************************************************/
unsigned char    number;
{
    unsigned char   digit;

    digit = number / 100;            ascii_buffer[0] = digit | 0x30;
    number = number - (digit * 100);
    digit = number / 10;             ascii_buffer[1] = digit | 0x30;
    number = number - (digit * 10);  ascii_buffer[2] = number | 0x30;
}
/***********************************************************************/
void report_error()
/***********************************************************************/
{
    printf("IEEE-488 ERROR - %04x (See METRABYTE manual)\n",IEEE_error);
    sound(1000); delay(250); nosound();
}


