/***********************************************************************
               F.W.BELL SERIES-9900 / MODEL-9500 GAUSSMETER
                      RS232 DEMONSTATION PROGRAM
************************************************************************
This program was written to assist users of a SERIES-9900 or MODEL-9500
Gaussmeter who wish to control the instrument remotely via the RS-232 
communications port.  This program was compiled with the Borland TURBO-C++
compiler.  The intent of this program is to provide the systems programmer
with a template to use during the development of his or her own code.
It also can be used as a test vehicle to verify the interface between the
computer and the Gaussmeter.

PROGRAM NOTES
-------------
1) The program is configured to operate on the PC's COM1 port, 9600 baud,
   odd parity, 1 stop bit and 7 data bits.  To change from COM1 to COM2
   change the "COMM" definition. To change the line parameters (baud,
   parity,etc) see the "SETTINGS" definition and refer to the "bioscom()"
   library function.
************************************************************************/

#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <bios.h>
#include <string.h>

/* definitions for Serial comm functions */

#define  ESC         27           /* <ESC> */
#define  CR          13           /* <CR>  */
#define  COMM        0            /* COM-1 port indicator */
                                  /*  (change to "1" for COM-2) */
#define  DATA_READY  0x0100       /* Port status, byte received, no errors */
#define  TX_READY    0x4000       /* Port status, tx ready */

#define  SETTINGS    (0x02 | 0x08 | 0x00 | 0xE0)   /* 0x02 = 7 data bits */
                                                   /* 0x00 = one stop bit */
                                                   /* 0x08 = odd parity bit */
                                                   /* 0xE0 = 9600 baud */
/* miscellaneous definitions */

#define  YES       1
#define  NO        0
#define  TIMER     2000000ul

/* Gaussmeter command strings */

char MEASURE1[12]    = {"ME100000000"};         /* measure */
char MODE1[5]        = {"MO12"};                /* mode select (Gauss DC) */
char RANGE1[5]       = {"RA14"};                /* range select (3 KG) */
char PEAK1_OFF[5]    = {"PE11"};                /* peak hold off */
char DISPLAY1[8]     = {"DI1222"};              /* display/bargraph/lite on */
char LOCK_ON[5]      = {"LO2"};                 /* lockon command */
char LOCK_OFF[5]     = {"LO1"};                 /* lockoff command */

char          GM_response[80];
int           outlen, inlen;
unsigned int  comm_error;

/* prototype functions */

void          GM_comm(), GM_setup(), GM_tx();
int           bioscom();

void main ()
{
    clrscr();
    printf("************************************************************************\n"  
           "               F.W.BELL SERIES-9900 / MODEL-9500 GAUSSMETER\n"  
           "                      RS232 DEMONSTATION PROGRAM\n"  
           "************************************************************************\n"  
           "This program was written to assist users of a SERIES-9900 or MODEL-9500\n"  
           " Gaussmeter who wish to control the instrument remotely via the RS-232 \n"  
           " communications port.  This program was compiled with the Borland TURBO-C++\n"  
           " compiler.  The intent of this program is to provide the systems programmer\n"
           " with a template to use during the development of his or her own code.\n"
           " It also can be used as a test vehicle to verify the interface between the\n"
           " computer and the Gaussmeter.\n\n"
           "This program requires that the Gaussmeter contain V1.7A or higher software.\n"
           " V1.8 contains improvements that will result in fewer errors and thus fewer\n"
           " retries.  The software version number is displayed immediately after \n"
           " power-up.\n\n"  
           " Either of the interface cables depicted in the Gaussmeter's User's manual\n"
           " may be used, but the cable WITHOUT handshaking is recommended for most\n"
           " applications.\n\n"
           "Press any key to continue.....");

    while (kbhit() != 0)  {getch();}
    while (kbhit() == 0);  getch(); clrscr();

    printf("The program is configured to operate on the PC's COM1 port, 9600 baud, odd\n"  
           " parity, 1 stop bit and 7 data bits.  See the comments in the source\n"  
           " file if you wish to change these parameters, or if any problems occur\n"
           " during execution.\n\n"  
           "At this point the GAUSSMETER should be configured to match these using the\n"  
           " COMMUNICATIONS FORMAT menu.  Place the Gaussmeter in the RUN mode before\n"  
           " starting the program.\n\n"
           "The program will begin by locking out the Gaussmeter's front panel and\n"
           " terminating the MASTER mode of operation.  Then the MODE is programmed for\n"
           " GAUSS-DC, the RANGE for 3 KG, PEAK HOLD off and DISPLAY DIGITS, BARGRAPH \n"
           " and BACKLIGHT on.  After this field measurements will be acquired and \n"
           " displayed until the user presses any key.  The program will work with a \n"
           " MODEL-9500 or with CHANNEL-1 on a SERIES-9900\n\n"  
           "Press any key to start.....");

    while (kbhit() == 0);  getch(); clrscr();

    /* Initialize serial comm Hardware and Gaussmeter */

    bioscom(0, SETTINGS, COMM);        
    printf("Locking out front panel...\n");
    GM_setup(&LOCK_ON[0], YES);
    GM_setup(&LOCK_ON[0], YES);

    /* Issue setup commands to Gaussmeter */

    printf("Programming MODE for Gauss-DC...\n");
    GM_setup(&MODE1[0], YES);     delay(3000);
    printf("Programming RANGE for 3 KG...\n");
    GM_setup(&RANGE1[0], YES);    delay(3000);
    printf("Programming PEAK HOLD off...\n");
    GM_setup(&PEAK1_OFF[0], YES); delay(3000);
    printf("Programming DISPLAY FORMAT, bargraph, digits and backlight on...\n");
    GM_setup(&DISPLAY1[0], YES);  delay(3000);

    /* Acquire field measurements */

    clrscr();
    printf("Acquiring field measurements from Gaussmeter...\n");
    printf("       Press any key to terminate.\n");
    delay(2000);
    while (kbhit() == 0) {GM_setup(&MEASURE1[0], NO);}

    /* Terminate session */ 

    printf("Unlocking front panel of Gaussmeter...\n");
    GM_setup(&LOCK_OFF[0], YES);
    delay(2000);
    clrscr();
}
/***********************************************************************/
void GM_setup(command_string,compare)
/***********************************************************************
This function initiates the transmission of a command to the Gauss-
meter and processes its response.

Upon entry <command_string> should contain the address of the ASCII 
buffer containing the command string to be transmitted.  <compare> should
be set to "1" if the caller wishes to have the entire response compared 
to the original command or "0" to compare just the first three characters
of the response.  The latter should be used when issuing the MEASURE, ZERO
or RELATIVE commands because the Gaussmeter will return real time field 
or status information in its response that probably won't match the 
original command string.

If the number of characters in the response is incorrect, or the contents 
of the response is incorrect (as determined by <compare>), or any hard-
ware errors occur the command will be reissued.  After 10 such attempts 
the process will be aborted and an error message will be printed to the 
screen.

Upon exit the global buffer <GM_response> will contain the ASCII response 
sent back by the Gaussmeter, terminated with a NULL character.  If any
errors occured that caused an abort the <GM_response> buffer will contain 
any characters received followed by a CR and NULL.  If a receiver error 
occured the global variable <comm_error> will indicate the type of error 
as follows:
                         D11 = 1: Framing Error
                         D10 = 1: Parity Error
                         D9  = 1: Overrun Error

Also upon exit the global variable <outlen> will indicate the total 
number of characters transmitted NOT including the ESC character but
including the CR character.  <inlen> will indicate the number of response
characters received including the <CR> character.  If everything went
right the two should be equal.
************************************************************************/
unsigned char *command_string, compare;
{
    int       check, retry = 10;
    do
    {
        GM_comm(command_string);
        if(compare == YES)
        {
            check = strncmp(command_string, GM_response, outlen - 1);
        }
        else
        {
            check = strncmp(command_string, GM_response, 3);
        }
        if(retry-- <= 0)
        {
            printf("Gaussmeter communications failure !\n");
            sound(1000); delay(500); nosound();
            break;
        }
        if(comm_error > DATA_READY) {printf("*** LINE ERROR = %04X ***\n",comm_error);}
        else
        {
            if(GM_response[0] == 0) {printf(" *** TIMEOUT ERROR ***\n");}
            else
            {
                if(inlen != outlen || check != 0)  {printf(" *** RESPONSE ERROR ***\n");}
            }
        }
        printf("Command sent = %s\t GM response = %s\n",command_string,GM_response);
    }   while (inlen != outlen || check != 0 || comm_error > DATA_READY);

 }
/***********************************************************************/
void GM_comm (command_string)
/***********************************************************************
This function handles the hardware interface between the PC and the 
Gaussmeter.  

Upon entry <command_string> should contain the address of the ASCII 
command string to be transmitted.

Upon exit the <GM_response> global buffer will contain the ASCII 
response sent back by the Gaussmeter, terminated with a <NULL> character.
If a receiver error occured (parity, framing or overrun), or a character 
is not received within a certain amount of time, or more than 79 
characters are received, <GM_response> will contain all prior characters 
received followed by a <CR> and <NULL>.  If a receiver error occured
the global variable <comm_error> will indicate the type of error as
follows:
                         D11 = 1: Framing Error
                         D10 = 1: Parity Error
                         D9  = 1: Overrun Error

Also upon exit the global variable <outlen> will indicate the total 
number of characters transmitted not including the <ESC> character but
including the <CR> character.  <Inlen> will indicate the number of 
characters received including the <CR> character.  If everything went
well the two should be equal.
************************************************************************/
unsigned char *command_string;
{
    unsigned char rx_byte = 0;
    long          timeout = TIMER;
    unsigned int  comm_status;

    inlen = 0;  comm_error = 0;

    /* Transmit command */

    GM_tx(command_string);

    /* Receive response.  Wait for each character to arrive and store it.
        If an error occured or too many characters arrived terminate the
        character string and quit. */

    while (rx_byte != CR && timeout-- > 0)
    {
        comm_status = 0x0F00 & (bioscom(3,0,COMM));
        if(comm_status >= DATA_READY)
        {
            rx_byte = bioscom(2,0,COMM);
            if ((inlen == 79) || (comm_status > DATA_READY)) 
            {
                rx_byte = CR;  comm_error |= comm_status;
            }
            GM_response[inlen] = rx_byte;
            inlen++; timeout = TIMER;
        }
    }
    GM_response[inlen] = 0;                          /* Terminate */
}
/***********************************************************************/
void GM_tx(command_string)
/***********************************************************************
This function transmits the <command_string> to the Gaussmeter, preceded
by an <ESC> character and terminated by the <CR> character.

Upon entry <command_string> should contain the address of the ASCII 
buffer containing the command string to be transmitted.
Also upon exit the global variable <outlen> will indicate the total 
number of characters transmitted not including the <ESC> character but
including the <CR> character.
************************************************************************/
    unsigned char  *command_string;
{
    unsigned char *command_save;

    command_save = command_string;

    while ((bioscom(3,0,COMM) & TX_READY) == 0);     /* Wait for TX ready */
    bioscom(1,ESC,COMM);                             /* Send <ESC>        */
    while (*command_string)
    {
        while ((bioscom(3,0,COMM) & TX_READY) == 0); /* Send command      */
        bioscom(1,*command_string++,COMM);
    }
    while ((bioscom(3,0,COMM) & TX_READY) == 0);     /* Wait for TX ready */
    bioscom(1,CR,COMM);                              /* Send <CR>         */

    outlen = command_string - command_save + 1;
}
